"""
Extends Wokkel's component to be able to dispatch to children methods
"""
from wokkel.component import Component
from twisted.words.protocols.jabber.jid import JID
from twisted.internet import defer


class XMPPComponent(Component):
	
	settings = {} # Share platform settings here

	def myJID( self ):
		if self.xmlstream:
			return self.xmlstream.thisEntity
	
	def remoteHost( self ):
		if self.xmlstream:
			return self.xmlstream.thisEntity.host.split(".",1)[1]

	def entityJID( self, entityName ):
		"""
		Returns the XMPP required JID for the entityName representing entity on platform
		"""
		if self.xmlstream:
			return JID( "%s@%s"%(entityName, self.xmlstream.thisEntity.host) )

	def runAMIFctryCommand( self, cmd, **kwargs ):
		fctry = self.parent.getServiceNamed("AMI")._connection.factory
		try:
			cmd = getattr( fctry, cmd )
		except:
			return defer.fail(Exception("Command not available"))
		return cmd( **kwargs )

	def runAMICommand( self, cmd, **kwargs ):
		"""
		Wrapper which returns AMI connection nicely
		"""
		conn = self.parent.getServiceNamed("AMI")._connection.factory.getConn()
		if not conn:
			raise Exception("No AMI Connection")
		try:
			cmd = getattr( conn, cmd )
		except:
			return defer.fail(Exception("Command not available"))
		return cmd( **kwargs )

	def handlerDispatch( self, modPath, className, method, **kw ):
		"""Dispatches request to dispatchers/children
		@param classPath: represents the class path for the object to check
		@param method: represents C{String} of the method to be called on the handler
		@param kw: key/val arguments to be passed on method
		"""
		for handler in self.handlers:
			mod = __import__( modPath, fromlist=[className] )
			kls = getattr( mod, className )
			if isinstance( handler, kls ):
				getattr( handler, method)( **kw ) # Run the method with name and params provided
				return True
		return False


