from time import time
from twisted.internet import reactor


class ConfDb( object ):

	config = {} # Will be populated here by tap module
	rooms = {} #{ confRoom:{chanid: [caller, usernum, time()]} }
	statusNotifiers = set()

	def registerStatusNotifier( self, notifierMethod ):
		self.statusNotifiers.add( notifierMethod )

	def unregisterStatusNotifier( self, notifierMethod ):
		self.statusNotifiers.discard( notifierMethod )

	def triggerEntityNotifies( self, confRoom ):
		"""Registered functions will be called to inform about changes inside specific confRoom"""
		for notifyFunc in self.statusNotifiers:
			reactor.callLater(self.config['gateway_delay'], notifyFunc, confRoom)

	def joinConf( self, confRoom, chanid, caller, usernum, atTime=None ):
		print "joinConf", confRoom, chanid, caller, usernum, atTime
		if not atTime:
			atTime = time()
		if confRoom in self.rooms:
			self.rooms[confRoom][chanid] = [caller, usernum, atTime]
		else: #First time, create dictionary structure behind
			self.rooms[confRoom] = { chanid: [caller, usernum, atTime] }
		self.triggerEntityNotifies( confRoom )

	def leaveConf( self, confRoom, chanid ):
		if confRoom in self.rooms and chanid in self.rooms[confRoom]:
			del self.rooms[confRoom][chanid]
			self.triggerEntityNotifies( confRoom )

	def endConf( self, confRoom ):
		if confRoom in self.rooms:
			self.rooms[confRoom] = {} #Empty confRoom
			self.triggerEntityNotifies( confRoom )

	def syncMembersFromListCmd( self, confRoom, membersData ):
		"""
		@example mbrsData: [[1, '101', 'SIP/101-00000003', 1361790916.0]]
		"""
		if confRoom not in self.rooms.keys():
			self.rooms[confRoom] = {}
		chanIds = [ mbrData[2] for mbrData in membersData ]
		# make sure the call with chanId was not already disconnected
		for storedChanId in self.rooms[confRoom].keys():
			if storedChanId not in chanIds:
				self.leaveConf( confRoom, storedChanId )
		for mbrData in membersData:
			if mbrData[2] not in self.rooms[confRoom].keys():
				self.joinConf( confRoom, mbrData[2], mbrData[1], mbrData[0], mbrData[3] )
		
	def getConfRooms( self ):
		"""
		Returns all the conference rooms in the database
		"""
		return self.rooms.keys()

	def getConfRoom( self, confRoom ):
		"""
		Returns call data in a specific conference room number
		@example return:
			['SIP/101-00000003', '101', '1', 1356606071.6245141]
		"""
		if confRoom not in self.rooms:
			return []
		return [ [chanId]+self.rooms[confRoom][chanId] for chanId  in self.rooms[confRoom] ]

	def addConfRoom( self, room ):
		if room not in self.rooms.keys():
			self.rooms[room] = {}


ConfDB = ConfDb()


