from twisted.internet import reactor

"""
-1 = Extension not found
0 = Idle
1 = In Use
2 = Busy
4 = Unavailable
8 = Ringing
16 = On Hold
"""

class ExtensDb (object):
	"""
	Cache for operating with entity presence information. Able to trigger notified methods in case of status changes
	"""

	extenStatus  = {}
	statusNotifiers  = set()

	def registerStatusNotifier( self, notifier ):
		"""External functions can use this method to register callback in case of extensions db changes"""
		self.statusNotifiers.add( notifier )

	def unregisterStatusNotifier( self, notifier ):
		"""Unregister from receiving notifies on database changes"""
		self.statusNotifiers.discard( notifierMethod )

	def triggerStatusNotifies( self, extension ):
		"""Registered functions will be called to inform about presence changes on specific extension"""
		for notifier in self.statusNotifiers:
			reactor.callLater(self.config['gateway_delay'], notifier, extension)

	def setExtenStatus( self, extension, status ):
		"""
		Updates extension status. Info coming normaly from AMI
		@param status: status info as coming from AMI (must be decoded at client side to support internationalisation
		"""
		self.extenStatus[extension] = status
		self.triggerStatusNotifies( extension )

	def getExtenStatus( self, extension ):
		"""
		@returns extension status
		"""
		try:
			return self.extenStatus[extension]
		except KeyError:
			return "-1" # Cope with what asterisk is reporting

	def getExtens( self ):
		return self.extenStatus.keys()
	

# Init extensDB data
ExtensDB = ExtensDb()
