import cPickle
from os.path import isfile

def fileData( fileName, data ):
	"""Dumps data to file, so it can survive across restarts"""
	# ToDo: Work with twisted.internet.fdesc here
	inFile = open(fileName, 'wb')
	cPickle.dump(data , inFile)
	inFile.close()

def loadData( fileName, data ):
	"""Loads the data from file dump
	@type fileName: C{String} representing the file path to load data from
	@type data: C{Dictionary} representing in memory database
	"""
	if not isfile( fileName ):
		return
	inFile = open(fileName, 'rb')
	fileData = cPickle.load(inFile)
	inFile.close()
	if fileData:
		data.update( fileData )


def iterDiff ( i1, i2 ):
	"""
	Finds out the elements which are in one list/tuple/iterator and not in the other
	returns: a C{Tuple} of two C{List}s with individual items
	"""
	s1 = set(i1)
	s2 = set(i2)
	hasL1 = [ d for d in s1 if d not in s2 ]
	hasL2 = [ d for d in s2 if d not in s1 ]
	return hasL1, hasL2


def entityFromLocation( location ):
	"""
	@example location: u'Local/102@from-queue/n'
		u'sip/104'
	"""
	import re
	m = re.match(r".+/(\d+).?", location)
	if m:
		return m.group(1)


def confRoomsFromConfigs( allConfigs ):
	"""
	Iterates over all configurations in asterisk and extracts from the app_meetme related ones the conference room numbers
	@returns a C{List} with all conf room numbers configured
	"""
	roomNrs = []
	for line in allConfigs:
		if line.startswith("app_meetme"): #@example: "app_meetme           /etc/asterisk/meetme_additional.conf    "
			config = line.split()[1]
			f = open(config, "rb")
			for line in f:
				if line.startswith("conf"): #@example: conf => 5000,
					room = line.split("=>")[1].strip().split(",")[0]
					roomNrs.append(room)
	return roomNrs


def confRoomParticipants( meetmeListRes ):
	"""
	@example good: ['User #: 01          101 Dan101               Channel: SIP/101-00000003     (unmonitored) 00:29:43', 'User #: 02          101 Dan101               Channel: SIP/101-00000004     (unmonitored) 00:21:07', '2 users in that conference.']
		with split: ['User', '#:', '01', '101', 'Dan101', 'Channel:', 'SIP/101-00000003', '(unmonitored)', '00:08:40']
	"""
	participants = []
	from datetime import datetime, timedelta
	import time
	for line in meetmeListRes:
		if line.startswith("User"):
			dataSplt = line.split()
			if len(dataSplt)!=9:
				continue #Something does not look right regarding data format
			pos = int(dataSplt[2])
			exten = dataSplt[3]
			chan = dataSplt[6]
			durDT = datetime.strptime(dataSplt[8],"%H:%M:%S")
			dur = timedelta(hours=durDT.hour, minutes=durDT.minute, seconds=durDT.second)
			tstmp = time.mktime((datetime.now()-dur).timetuple())
			participants.append( [pos, exten, chan, tstmp] )
	return participants
