from twisted.python import usage, log
from twisted.application import internet, service
from twisted.words.protocols.jabber.jid import JID

from wokkel.ping import PingHandler
from wokkel.disco import DiscoHandler
from mbitrans import ami, pinger, iq, component, presence
from mbitrans.pubserv import PubServHandler
from mbitrans.cmd import CmdHandler

from mbitrans.settingsdb import SettingsDB
from mbitrans.extensdb import ExtensDB
from mbitrans.callsdb import CallsDB
from mbitrans.presencedb import PresenceDB
from mbitrans.extenpermsdb import ExtenPermsDB
from mbitrans.parkdb import ParkDB
from mbitrans.confdb import ConfDB
from mbitrans.mailboxdb import MailboxDB
from mbitrans.queuedb import QueueDB
from mbitrans.permsdb import PermsDB
from mbitrans.config import FileConfig


import logging
logging.basicConfig()

class Options(usage.Options):

	optFlags = [
		["verbose", "v", "Start server with verbosity on - useful for troubleshooting"],
	]

	optParameters = [
		['ami_host', None, '127.0.0.1', 'The host where to reach asterisk'],
		['ami_port', None, 5038, 'The port where to reach asterisk', usage.portCoerce],
		['ami_user', None, 'openfire', 'Username to use when connecting to asterisk'],
		['ami_pwd', None, 'mbit2010', 'Password to use when connecting to asterisk'],
		['ami_sync', None, 600, 'Interval in seconds to resync cache from asterisk, 0 disables it'],
		['ami_callstats', None, 3, 'Interval in seconds to refresh call information, 0 disables it'],
		['xmpp_host', None, '127.0.0.1', 'The host where to reach XMPP server'],
		['xmpp_port', None, 5275, 'The port where to reach XMPP server', usage.portCoerce],
		['xmpp_secret', None, 'abc123', 'The secret to use an external component to XMPP server'],
		['xmpp_sdomain', 'i', 'ast1', 'The subdomain to register with as component to XMPP server'],
		['xmpp_ping', None, 300, 'Interval in seconds to send pings towards XMPP server'],
		['gateway_delay', None, 0.5, 'Seconds to wait before passing messages between protocols'],
		['config', 'f', '/etc/mbitrans/mbitrans.cfg', 'Path to configuration file']
	]


def makeService(options):

	cfg = FileConfig(options).parse()
	
	shareConfig( cfg )


	ms = service.MultiService()

	#start Asterisk connection
	amiSrv = internet.TCPClient( cfg['ami_host'], cfg['ami_port'], ami.MBitAMIFactory(cfg) )
	amiSrv.setName("AMI")
	amiSrv.setServiceParent(ms)


	#connect to XMPP server
	#xmppComponent.handlerDispatch( 'mbitrans.iq','IQHandler', 'sayYes' )
	xmppComponent = component.XMPPComponent(cfg['xmpp_host'], cfg['xmpp_port'], cfg['xmpp_sdomain'], cfg['xmpp_secret'])
	xmppComponent.setName( "XMPP" )
	xmppComponent.setServiceParent(ms)
	pinger.Pinger(JID("mbitdev"), JID("ast1.mbitdev"), cfg['xmpp_ping']).setHandlerParent(xmppComponent)
	presence.PresenceHandler().setHandlerParent(xmppComponent)
	iq.IQHandler().setHandlerParent(xmppComponent)
	PubServHandler().setHandlerParent(xmppComponent)
	DiscoHandler().setHandlerParent(xmppComponent)
	CmdHandler().setHandlerParent(xmppComponent)

	return ms


def shareConfig(settings):
	"""Makes  the platform settings available to other subcomponents"""
	for smod in (CallsDB, ExtensDB, PresenceDB, ExtenPermsDB, SettingsDB, QueueDB, MailboxDB, PermsDB, ParkDB, ConfDB):
		smod.config = settings
