from time import time
from twisted.python import log
from twisted.internet import reactor

from mbitrans.libmbit import iterDiff
from mbitrans.callsdb import CallsDB


class ParkDb( object ):
	"""I am responsible to keep a cache of parked calls on the platform
	Not keeping reference towards uniqueid since sync does not contain this info
	"""
	config = {} # Will be populated here by tap module
	parkedCalls = {} # Indexed calls per callids, {'channelid':[parklot,parkexten,caller,called,timeout,timestamp,extension]}
	statusNotifiers = set() #They are all informed when parked calls changes

	def registerStatusNotifier( self, notifierMethod ):
		"""External functions can use this method to register callback in case of parked calls list changes"""
		self.statusNotifiers.add( notifierMethod )

	def unregisterStatusNotifier( self, notifierMethod ):
		"""Unregister from receiving notifies on specific entity call status change"""
		self.statusNotifiers.discard( notifierMethod )

	def triggerNotifies( self ):
		"""Registered functions will be called to inform about status changes on specific entityName"""
		for notifyFunc in self.statusNotifiers:
			reactor.callLater(self.config['gateway_delay'], notifyFunc)

	def parked( self, amiChanInfo ):
		print "Parked received, data:", amiChanInfo
		calling = amiChanInfo['calleridnum']
		called = ''
		if amiChanInfo['connectedlinenum'] != '<unknown>': #Special case when destination cannot be set, eg: dialing directly park
			called = amiChanInfo['connectedlinenum']
		if amiChanInfo['channel'] in CallsDB.activeCalls and CallsDB.activeCalls[amiChanInfo['channel']][7]=="B":
			calling, called = called, calling #Swap since Asterisk is reporting them wrongly
		extension = ""
		if amiChanInfo['channel'] in CallsDB.activeCalls:
			if CallsDB.activeCalls[amiChanInfo['channel']][7]=="A":
				extension=CallsDB.activeCalls[amiChanInfo['channel']][0]
			elif CallsDB.activeCalls[amiChanInfo['channel']][7]=="B":
				extension=CallsDB.activeCalls[amiChanInfo['channel']][1]
		self.parkedCalls[amiChanInfo['channel']] = [ amiChanInfo['parkinglot'], amiChanInfo['exten'],
				calling, called, amiChanInfo['timeout'], time(), extension ]
		print "parkdb:", self.parkedCalls
		self.triggerNotifies()

	def unparked( self, channel ):
		if channel in self.parkedCalls:
			del self.parkedCalls[channel]
			self.triggerNotifies()

	def syncParkedCalls( self, amiParked ):
		"""
		Called by AMI with active parked info retreieved from parked status command
		@param amiParked: list of parked channels info as it comes originally from ami
		"""
		amiChanIds = [ amiChan['channel'] for amiChan in amiParked ]
		outCachedChans, newAmiChans = iterDiff( self.parkedCalls.keys(), amiChanIds )
		for outChan in outCachedChans:
			self.unparked(outChan)
		for newChan in newAmiChans:
			for amiPark in amiParked:
				if amiPark['channel'] == newChan:
					self.parked( amiPark )

	def getParkedCalls( self ):
		"""
		Returns details of all parked calls
		{'SIP/101-00000016': ['default', '701', '101', '<unknown>', '45', 1356532285.5320251]}
		@returns: list of parked calls in form of (one parked call):
			['SIP/101-00000017', 'default', '701', '101', '<unknown>', '45', 1356532285.532025]
		"""
		return [ [chanid]+self.parkedCalls[chanid] for chanid  in self.parkedCalls ]
		
ParkDB = ParkDb()
