from wokkel import xmppim
from twisted.words.protocols.jabber.jid import JID
from twisted.python import log
from twisted.internet import reactor

from mbitrans.permsdb import PermsDB
from mbitrans.presencedb import PresenceDB


class PresenceHandler(xmppim.PresenceProtocol):

	def connectionInitialized(self):
		"""
		Called when connection is initialized. Usefull to send initial presence info
		"""
		xmppim.PresenceProtocol.connectionInitialized(self)
		PermsDB.registerStatusNotifier( "*", self.triggeredUserPerms )
		self.probeUsersPresence()

	def probeUsersPresence( self ):
		for user in PermsDB.getAllUsers():
			self.probe(JID(user))

	def triggeredUserPerms( self, entityTag, user, action ):
		"""
		@param entityTag: entity category
		@param user: user in username@domain format
		@param action: one of "set" or "remove"
		"""
		#if action == "remove" and not PermsDB.getUserEntityTags(user): #User was removed, it does not longer have any entity perms in db
		#	PresenceDB.removeUser( user )
		reactor.callLater( 1, self.probe, JID(user) ) #Give a bit of time for roster to build in openfire

	def availableReceived(self, presence):
		"""
		Called when presence is received from XMPP side
		"""
		PresenceDB.setUserPresence( presence.sender.userhost(), 'available', presence.show, "" )

	def unavailableReceived(self, presence):
		"""
		Called when presence is received from XMPP side
		"""
		PresenceDB.setUserPresence( presence.sender.userhost(), 'unavailable', presence.show, "" )
