from twisted.internet import reactor

from mbitrans import libmbit


class PresenceDb (object):
	"""
	Cache for operating with presence information for XMPP users. 
	Able to trigger notified methods in case of status changes
	@param presenceData: holds the presence cache
	"""
	config = {} #platform settings, populated here by tap file
	presenceData  = {}  # key string, value list of [type, show, status]
	statusNotifiers  = set() # list of methods to call on triggers

	def registerStatusNotifier( self, notifier ):
		"""External functions can use this method to register callback in case of presence db changes"""
		self.statusNotifiers.add( notifier )

	def unregisterStatusNotifier( self, notifier ):
		"""Unregister from receiving notifies on database changes"""
		self.statusNotifiers.discard( notifierMethod )

	def triggerStatusNotifies( self, user ):
		"""Registered functions will be called to inform about presence changes on specific user"""
		for notifier in self.statusNotifiers:
			reactor.callLater(self.config['gateway_delay'], notifier, user)

	def setUserPresence( self, user, typ, show, status ):
		"""
		Set/Updates a user presence.
		@param typ: type of presence - available/unavailable
		@param show: presence show information
		@param status: free text description of the presence info
		"""
		self.presenceData[user] = [typ, show, status]
		self.triggerStatusNotifies(user)

	def removeUser( self, user ):
		if user in self.presenceData.keys():
			del self.presenceData[user]
		self.triggerStatusNotifies(user)

	def getUserPresence( self, user ):
		"""
		@returns a tuple of: type, show, status
		"""
		try: 
			return self.presenceData[user]
		except KeyError:
			return ('','','') #No presence info

	def getUsers( self ):
		return self.presenceData.keys()
	

# Init presenceDB data
PresenceDB = PresenceDb()
