from time import time
from mbitrans.extensdb import ExtensDB
from twisted.internet import reactor

"""
MemberStatus:
As far as I know Possible values are:
/*! Device is valid but channel didn't know state */
define AST_DEVICE_UNKNOWN	0
/*! Device is not used */
define AST_DEVICE_NOT_INUSE	1
/*! Device is in use */
define AST_DEVICE_INUSE	2
/*! Device is busy */
define AST_DEVICE_BUSY	 3
/*! Device is invalid */
define AST_DEVICE_INVALID	4
/*! Device is unavailable */
define AST_DEVICE_UNAVAILABLE	5
/*! Device is ringing */
define AST_DEVICE_RINGING	6
/*! Device is ringing *and* in use */
define AST_DEVICE_RINGINUSE	7
/*! Device is on hold */
define AST_DEVICE_ONHOLD	8
"""

class QueueDb( object ):
	config = {} # Will be populated here by tap module
	queued = {} # example: self.queued[queue] = {chanid: [position, caller, wait, time] }
	agents = {} # example: self.agents[queue] = {agent: [membership, status, location, callstaken, lastcall, paused, penalty] }
		    # {'200': {'Local/102@from-queue/n': ['dynamic', '1', 'Local/102@from-queue/n', '0', '0', '0', '0']}}
		    # {'200': {'Local/102@from-queue/n': ['static', '5', 'Local/102@from-queue/n', '0', '0', '0', '0']}}
	stats = {} # example: self.stats[queue] = {'servicelevelperf': '0.0', 'talktime': '0', 'calls': '2', 'max': '0', 'completed': '0', 
				#'servicelevel': '60', 'weight': '0', 'strategy': 'ringall',  'holdtime': '0', 'abandoned': '9'}

	statusNotifiers = {
		'queued': set(), #@example: 'queued':{ queue: set([method, method2]) }
		'agents': set(),
		'stats': set()
		}

	#def __init__( self ):
	#	ExtensDB.registerStatusNotifier( self.onExtenStatusChange ) #Used to cope with bug of asterisk not refreshing agents/members info

	def registerStatusNotifier( self, subject, notifierMethod ):
		"""
		@param subject: one of queued, agents, stats
		"""
		self.statusNotifiers[subject].add( notifierMethod )

	def unregisterStatusNotifier( self, subject, notifierMethod ):
		self.statusNotifiers.discard( notifierMethod )

	def triggerEntityNotifies( self, subject, queue ):
		"""Registered functions will be called to inform about changes inside specific confRoom"""
		for notifyFunc in self.statusNotifiers[subject]:
			reactor.callLater(self.config['gateway_delay'], notifyFunc, queue)

	def join( self, queue, position, caller, chanid, wait ):
		if queue not in self.queued:
			self.queued[queue] = {chanid: [position, caller, wait, time()]}
		elif chanid not in self.queued[queue] or self.queued[queue][chanid][:-1] != [position, caller, wait]:
			self.queued[queue][chanid] = [position, caller, chanid, wait]
		self.triggerEntityNotifies( 'queued', queue )


	def leave( self, queue, chanid ):
		if queue in self.queued and chanid in self.queued[queue]:
			del self.queued[queue][chanid]
			self.triggerEntityNotifies( 'queued', queue )
			if len(self.queued[queue])==0:
				del self.queued[queue] #cleanup so deleted queues do not stay unecessary in memory here


	def setAgent( self, queue, agent, membership, status, location, callstaken, lastcall, paused, penalty ):
		if queue not in self.agents.keys():
			self.agents[queue] = { agent: [membership, status, location, callstaken, lastcall, paused, penalty] }
		elif agent not in self.agents[queue]:
			self.agents[queue][agent] = [membership, status, location, callstaken, lastcall, paused, penalty]
		elif self.agents[queue][agent] != [membership, status, location, callstaken, lastcall, paused, penalty]:
			self.agents[queue][agent] = [membership, status, location, callstaken, lastcall, paused, penalty]
		self.triggerEntityNotifies( 'agents', queue )

	def rmAgent( self, queue, agent ):
		if queue in self.agents and agent in self.agents[queue]:
			del self.agents[queue][agent]
			self.triggerEntityNotifies( 'agents', queue )
			if len(self.agents[queue])==0:
				del self.agents[queue]
	def pauseAgent( self, queue, agent, paused ):
		if queue in self.agents and agent in self.agents[queue]:
			self.agents[queue][agent][5] = paused
			self.triggerEntityNotifies( 'agents', queue )
		

	def setStats( self, queue, stats ):
		"""
		@type stats: A C{Dictionary} of stats data 
		"""
		self.stats[queue] = stats
		self.triggerEntityNotifies( 'stats', queue )

	def getQueueCalls( self, queue ):
		"""
		@example return:
		"""
		if queue in self.queued:
			return [ [chanid]+self.queued[queue][chanid] for chanid  in self.queued[queue] ]
		return []

	def getQueueAgents( self, queue ):
		"""
		@returns: all agents on a queue together with their details
		"""
		if queue in self.agents:
			return [ [agent]+self.agents[queue][agent] for agent  in self.agents[queue] ]
		return []

	def getAllAgents( self ):
		"""
		@returns: all agents in the system, independent of the queue they are registered with
		"""
		agents = set()
		for queue in self.agents.keys():
			agents.update( self.agents[queue].keys() )
		return agents

	def getQueueStats( self, queue ):
		"""
		@example return:
		"""
		if queue in self.stats:
			return self.stats[queue]
		return {}

	def getQueues( self ):
		return self.stats.keys()



QueueDB = QueueDb()
