"""
Cache here useful settings
"""

from twisted.internet import reactor


class SettingsDb( object ):
	entities = {}

	statusNotifiers  = set()

	def registerStatusNotifier( self, notifier ):
		"""External functions can use this method to register callback in case of extensions db changes"""
		self.statusNotifiers.add( notifier )

	def unregisterStatusNotifier( self, notifier ):
		"""Unregister from receiving notifies on database changes"""
		self.statusNotifiers.discard( notifierMethod )

	def triggerStatusNotifies( self, entity, key ):
		"""Registered functions will be called to inform about setting changes on specific extension and key"""
		for notifier in self.statusNotifiers:
			reactor.callLater(self.config['gateway_delay'], notifier, entity, key)

	def setEntity( self, entity, key, val ):
		if entity not in self.entities:
			self.entities[entity] = {key:val}
			self.triggerStatusNotifies( entity, key )
		elif key not in self.entities[entity] or self.entities[entity][key]!=val:
			self.entities[entity][key] = val
			self.triggerStatusNotifies( entity, key )

	def unsetEntity( self, entity ):
		"""
		Used in case of no longer monitoring an entity or entity gets deleted on asterisk side
		@ToDo: Use this mechanism since it is not used anywhere right now
		"""
		if entity in self.entities:
			del self.entities[entity]
			self.triggerStatusNotifies( entity, 'unset' )

	def getEntityProperty( self, entity, key, default ):
		"""
		@param defaut: used in case of no key found
		"""
		try:
			return self.entities[entity][key]
		except KeyError:
			return default

	def getEntityForProperty( self, key, val, default ):
		"""
		Reverse searches an entity with key, val matching
		"""
		for entity in self.entities.keys():
			if key in self.entities[entity] and self.entities[entity][key] == val:
				return entity
		return default

	def getPropertyValues( self, key ):
		"""
		Return distinct values for the key. Useful for example to return all voicemail ids in the system
		"""
		values = set()
		for entity in self.entities.keys():
			if key in self.entities[entity].keys():
				values.add(self.entities[entity][key])
		return values

	def getEntityForVm( self, vmid ):
		if vmid in self.mbox2entity:
			return self.mbox2entity[vmid]


SettingsDB = SettingsDb()

